@@ -279,6 +279,14 @@ class DataContext(object):
         """
         return os.path.join(self._get_local_context_dir(), constants._MANAGED_OBJECTS)
 
+    @staticmethod
+    def s3_remote_from_url(remote_ctxt_url):
+        """ remove '/context' """
+        if remote_ctxt_url is None:
+            return 'None'
+        else:
+            return remote_ctxt_url[:-len('/context')]
+
     def get_remote_object_dir(self):
         """
         Where objects live on remote.
@@ -298,10 +306,8 @@ class DataContext(object):
 
     def init_remote_db(self):
         """
-        NOTE: Not sure what this was supposed to do...
-
-        Returns:
-
+        Currently a no-op.  Will connect to something like dynamodb
+        when we have external indices for objects in cloud storage (aka S3).
         """
         pass
 
@@ -504,6 +510,15 @@ class DataContext(object):
         #print "frames {}".format(frames)
         #print "auths {}".format(auths)
 
+    def bundle_count(self):
+        """ Determine how many bundles in the current local context
+        Returns:
+            (int): Count of bundles in this local context
+        """
+        assert self.local_engine is not None
+
+        return hyperframe.bundle_count(self.local_engine)
+
     def dbck(self):
         """
         Do a database check.
